#
# Name       History
# Ken        Initiated
#
#
# Contact Info:
# <ken_zhai@sdesigns.com>
#
# Sigma Designs
# 1778 McCarthy BLVD
# Milpitas CA 95035, USA
#
####################################################################################################
#
# build binutils for use on the target system
#
####################################################################################################
BINUTILS_VERSION:=$(strip $(subst ",, $(BR2_BINUTILS_VERSION)))
#"))
ifeq ($(strip $(DOWNLOAD_SITE_PUBLIC)),y)
BINUTILS_SITE:=http://ftp.gnu.org/gnu/binutils
else
BINUTILS_SITE:=$(DOWNLOAD_SITE)
endif
BINUTILS_SOURCE:=binutils-$(BINUTILS_VERSION).tar.bz2
BINUTILS_DIR:=$(BUILD_DIR)/binutils-$(BINUTILS_VERSION)
BINUTILS_CAT:=bzcat

$(DL_DIR)/$(BINUTILS_SOURCE):
	@[ -f $@ ] || $(WGET) -P $(@D) $(BINUTILS_SITE)/$(@F)

$(BINUTILS_DIR)/.unpacked: $(DL_DIR)/$(BINUTILS_SOURCE)
	@[ -d $(@D) ] || \
	($(BINUTILS_CAT) $< | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -; \
	touch $@; \
	)

$(BINUTILS_DIR)/.patched: $(BINUTILS_DIR)/.unpacked
	@cd $(TOPDIR); $(TOPDIR)/toolchain/patch-kernel.sh $(@D) $(SUBDIR)/$(BINUTILS_VERSION) binutils-$(BINUTILS_VERSION)-\*.patch
	touch $@
.PHONY: $(ROOTFS_TMP_DIR)
$(BINUTILS_DIR)/.configured:  $(ROOTFS_TMP_DIR) $(BINUTILS_DIR)/.patched
	@[ -f $@ ] || \
	(cd $(@D); rm -rf config.cache; \
		CC=$(TARGET_CC) \
		USE_INCLUDED_LIBINTL=yes \
                $(TARGET_CONFIGURE_OPTS) \
                $(TARGET_CONFIGURE_ARGS) \
		CFLAGS_FOR_BUILD="-O2" \
                CFLAGS="-fPIC $(TARGET_CFLAGS) -I$(TOOLCHAIN_INCLUDE_PATH)/include -L$(TOOLCHAIN_LIBS_PATH)/lib" \
		./configure \
		--host=$(GNU_TARGET_NAME) \
		--target=$(GNU_TARGET_NAME) \
		--build=$(GNU_HOST_NAME) \
		--prefix=$</usr \
		--exec-prefix=$</usr \
		--bindir=$</usr/bin \
		--sbindir=$</usr/sbin \
		--sysconfdir=$</etc \
		--datadir=$</usr/share \
		-localstatedir=$</var \
		--mandir=$</usr/man \
		--infodir=$</usr/info \
		--libdir=$</lib \
		--enable-shared; \
	touch $@; \
	)

binutils-dependancies: $(ROOTFS_TMP_DIR)/lib $(ROOTFS_TMP_DIR)/include
	@if ! which bison > /dev/null ; then \
		echo -e "\n\nYou must install 'bison' on your build machine\n"; \
		exit 1; \
	fi; \
	if ! which flex > /dev/null ; then \
		echo -e "\n\nYou must install 'flex' on your build machine\n"; \
		exit 1; \
	fi; \
	if ! which msgfmt > /dev/null ; then \
		echo -e "\n\nYou must install 'gettext' on your build machine\n"; \
		exit 1; \
	fi; \
	for x in $(TARGET_DIR)/bin $(TARGET_DIR)/lib $(ROOTFS_LIB) $(ROOTFS_BIN) $(ROOTFS_SBIN) $(ROOTFS_USR) $(ROOTFS_INC); \
	do \
	( \
	[ -d $$x ] || mkdir -p $$x; \
	); \
	done

$(BINUTILS_DIR)/binutils/objdump: $(BINUTILS_DIR)/.configured
	$(MAKE) BUILD_CC=$(TARGET_CC) HOSTCC=$(HOSTCC) CC=$(TARGET_CC) \
	BUILD_CCFLAGS="-I$(<D) -I$(<D)/include" \
	BUILD_LDFLAGS="" DESTDIR=$(ROOTFS_TMP_DIR) -C $(<D); \
	touch -c $@

$(ROOTFS_TMP_DIR)/lib $(ROOTFS_TMP_DIR)/include:
	@[ -d $@ ] || mkdir -p $@; \

$(ROOTFS_TMP_DIR)/lib/libintl.so:
	@find $(@D) -name "$(@F)*" -print0 | xargs -0 /bin/rm -f; \
	cp -f $(BINUTILS_DIR)/intl/$(@F) $(@D); \
	cd $(@D); ln -s $(@F) $(@F).0; ln -s $(@F).0 $(@F).0.0; \
	touch -c $@

$(ROOTFS_TMP_DIR)/lib/libiberty.a:
	@cp -f $(BINUTILS_DIR)/libiberty/$(@F) $(@D); \
	touch -c $@

$(ROOTFS_TMP_DIR)/lib/libiberty.so:
	@find $(@D) -name "$(@F)*" -print0 | xargs -0 /bin/rm -f; \
	cp -f $(BINUTILS_DIR)/libiberty/$(@F) $(@D); \
	cd $(@D); ln -s $(@F) $(@F).0; ln -s $(@F).0 $(@F).0.0; \
	touch -c $@

$(ROOTFS_TMP_DIR)/lib/libopcodes-2.18.so:
	@find $(@D) -name "$(@F)*" -print0 | xargs -0 /bin/rm -f; \
	cp -f $(BINUTILS_DIR)/opcodes/.libs/$(@F) $(@D); \
	cd $(@D); ln -s $(@F) $(@F).0; ln -s $(@F).0 $(@F).0.0; \
	touch -c $@

$(ROOTFS_TMP_DIR)/lib/libbfd-2.18.so: $(ROOTFS_TMP_DIR)/lib/libopcodes-2.18.so
	@find $(@D) -name "$(@F)*" -print0 | xargs -0 /bin/rm -f; \
	cp -f $(BINUTILS_DIR)/bfd/.libs/$(@F) $(@D); \
	cd $(@D); ln -s $(@F) libbfd.so.0; ln -s libbfd.so.0 libbfd.so; \
	touch -c $@

$(ROOTFS_TMP_DIR)/include/bfd.h:
	@for x in bfd/bfd.h include/ansidecl.h /include/symcat.h include/libiberty.h; \
	do \
	( \
	cp -f $(BINUTILS_DIR)/$$x $(@D); \
	); \
	done

$(ROOTFS_TMP_DIR)/lib/libbfd.a: $(ROOTFS_TMP_DIR)/lib/libintl.a $(ROOTFS_TMP_DIR)/lib/libiberty.a
	@[ ! -f $(BINUTILS_DIR)/bfd/.libs/$(@F) ] || cp -f $(BINUTILS_DIR)/bfd/.libs/$(@F) $(@D)
	@touch -c $@

$(TARGET_DIR)/lib/libintl.so: $(ROOTFS_TMP_DIR)/lib/libintl.so
	@find $(@D) \( -name "$(@F)*" \) -exec rm -f {} \;; \
	cp -f $< $(@D); \
	(cd $(@D); ln -s $(<F) $(<F).0; ln -s $(<F).0 $(<F).0.0); \
	touch -c $@
ifeq '$(filter debug,$(COMPILKIND))' ''
	@$(STRIP) --strip-unneeded $@.0.0
endif

$(TARGET_DIR)/lib/libopcodes-2.18.so: $(ROOTFS_TMP_DIR)/lib/libopcodes-2.18.so
	@find $(@D) \( -name "$(@F)*" \) -exec rm -f {} \;; \
	cp -f $< $@; \
	(cd $(@D); ln -s $(<F) $(<F).0; ln -s $(<F).0 $(<F).0.0); \
	touch -c $@
ifeq '$(filter debug,$(COMPILKIND))' ''
	@$(STRIP) --strip-unneeded $@
endif

$(TARGET_DIR)/lib/libbfd-2.18.so: $(ROOTFS_TMP_DIR)/lib/libbfd-2.18.so
	@find $(@D) \( -name "$(@F)*" \) -exec rm -f {} \;; \
	cp -f $< $@; \
	(cd $(@D); ln -s $(<F) $(<F).0; ln -s $(<F).0 $(<F).0.0); \
	touch -c $@
ifeq '$(filter debug,$(COMPILKIND))' ''
	@$(STRIP) --strip-unneeded $@
endif

$(TARGET_DIR)/lib/libiberty.so: $(ROOTFS_TMP_DIR)/lib/libiberty.so
	@find $(@D) \( -name "$(@F)*" \) -exec rm -f {} \;; \
	cp -f $< $(@D); \
	(cd $(@D); ln -s $(<F) $(<F).0; ln -s $(@F).0 $(@F).0.0); \
	touch -c $@
ifeq '$(filter debug,$(COMPILKIND))' ''
	@$(STRIP) --strip-unneeded $@.0.0	
endif

binutils: binutils-dependancies $(TARGET_DIR)/bin/objdump $(ROOTFS_TMP_DIR)/include/bfd.h binutils-libs
test:
	@BUILD_CC=$(TARGET_CC) HOSTCC=$(HOSTCC) CC=$(TARGET_CC) \
	$(MAKE) \
		prefix=$(@D)/.. \
		exec_prefix=$(@D)/.. \
		bindir=$(@D)/../bin \
		sbindir=$(@D)/../sbin \
		libexecdir=$(@D)/../lib \
		datadir=$(@D)/../usr/share \
		sysconfdir=$(@D)/../etc \
		localstatedir=$(@D)/../var \
		libdir=$(@D)/../lib \
		infodir=$(@D)/../info \
		mandir=$(@D)/../man \
		includedir=$(@D)/../include \
		gxx_include_dir=$(@D)/../include/c++ \
		ticdir=$(@D)/../usr/share/terminfo \
		tooldir=$(@D)/../usr \
		build_tooldir=$(@D)/../usr \
		DESTDIR=$(@D)/.. \
		-C $(<D)/.. install

$(TARGET_DIR)/bin/objdump: $(BINUTILS_DIR)/binutils/objdump
	@[ ! -f $(BINUTILS_DIR)/binutils/.libs/objdump ] || cp -f $(BINUTILS_DIR)/binutils/.libs/objdump $(@D)
ifeq '$(filter debug,$(COMPILKIND))' ''	
	@[ ! -f $(BINUTILS_DIR)/binutils/.libs/objdump ] || $(STRIP) --strip-unneeded $(BINUTILS_DIR)/binutils/.libs/objdump > /dev/null 2>&1
	@[ ! -f $@ ] || $(STRIP) --strip-unneeded $@ > /dev/null 2>&1
endif

binutils-source: $(DL_DIR)/$(BINUTILS_SOURCE)

.PHONY: binutils-clean binutils-uninstall binutils-install binutils-dirclean

binutils-clean: binutils-uninstall
	@[ ! -d $(BINUTILS_DIR) ] || $(MAKE) -C $(BINUTILS_DIR) clean
	@[ ! -f $(BINUTILS_DIR)/intl/libintl.so.0.0 ] || $(RM) $(BINUTILS_DIR)/intl/libintl.so.0.0
	@[ ! -f $(BINUTILS_DIR)/libiberty/libiberty.so.0.0 ] || $(RM) $(BINUTILS_DIR)/libiberty/libiberty.so.0.0

binutils-libs: $(TARGET_DIR)/lib/libbfd-2.18.so $(TARGET_DIR)/lib/libopcodes-2.18.so $(TARGET_DIR)/lib/libintl.so $(TARGET_DIR)/lib/libiberty.so
binutils-install: binutils

binutils-uninstall:
	@for x in $(TARGET_DIR)/bin/objdump $(ROOTFS_TMP_DIR)/bin/objdump; \
	do \
	( \
	[ ! -f $$x ] || rm -f $$x; \
	); \
	done; \
	for x in $(ROOTFS_TMP_DIR)/lib $(TARGET_DIR)/lib; \
	do \
	( \
	[ ! -d $$x ] || find $$x/ -name "libintl*" -print0 | xargs -0 /bin/rm -f; \
	[ ! -d $$x ] || find $$x/ -name "libiberty*" -print0 | xargs -0 /bin/rm -f; \
	[ ! -d $$x ] || find $$x/ -name "libbfd*" -print0 | xargs -0 /bin/rm -f; \
	[ ! -d $$x ] || find $$x/ -name "libopcodes*" -print0 | xargs -0 /bin/rm -f; \
	); \
	done

binutils-dirclean: binutils-uninstall
	@-rm -rf $(BINUTILS_DIR)
