
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>

/* Default loop counts */
#define DEF_LOOP1_CNT	10000000
#define DEF_LOOP2_CNT	(DEF_LOOP1_CNT * 10)

/* Loop element -- common element to be shared for all loop functions */
static inline void loop_element(void)
{
	struct timeval tvx;
	struct timezone tzx;
	int resx;
	if ((resx = gettimeofday(&tvx, &tzx)) != 0) 
		printf("%d: gettimeofday error (%d)\n", __LINE__, resx);
	return;
}

/* Calculate the elapse time, in turn of seconds (with micro-second precision) */
static inline void diff_timeval(struct timeval *tv1, struct timeval *tv2, 
					time_t *sec, suseconds_t *usec)
{
	*sec = *usec = 0;							
	if (tv2->tv_usec < tv1->tv_usec) {
		*sec = tv2->tv_sec - tv1->tv_sec - 1;
		*usec = (1000000 + tv2->tv_usec) - tv1->tv_usec;
	} else {
		*sec = tv2->tv_sec - tv1->tv_sec;	
		*usec = tv2->tv_usec - tv1->tv_usec;
	}
}

/* Macro to build the loop function */
#define BUILD_LOOP_FUNC(name)							\
static int name##_func(unsigned long long cnt, time_t *sec, suseconds_t *usec)	\
{										\
	unsigned long long i;							\
	struct timeval tv1, tv2;						\
	struct timezone tz1, tz2;						\
	int res;								\
	printf("Running %s with loop count %lld\n", __FUNCTION__, cnt);		\
	if ((res = gettimeofday(&tv1, &tz1)) != 0) {				\
		printf("%d: gettimeofday error (%d)\n", __LINE__, res);		\
		return -1;							\
	}									\
	for (i = 0; i < cnt; i++) 						\
		loop_element();							\
	if ((res = gettimeofday(&tv2, &tz2)) != 0) {				\
		printf("%d: gettimeofday error (%d)\n", __LINE__, res);		\
		return -1;							\
	}									\
	diff_timeval(&tv1, &tv2, sec, usec);					\
	return 0;								\
}

BUILD_LOOP_FUNC(loop1);	/* instance of loop function: loop1_func */
BUILD_LOOP_FUNC(loop2);	/* instance of loop function: loop2_func */

/* Main starts here */
int main(int argc, char *argv[])
{
	unsigned long long loop1_cnt = DEF_LOOP1_CNT;
	unsigned long long loop2_cnt = DEF_LOOP2_CNT;
	time_t sec;
	suseconds_t usec;

	if (argc > 3) 
		goto usage;
	else {
		/* Parsing options (may be overriding default loop counts) */
		switch(argc) {
			case 3: loop2_cnt = (unsigned long long)atoll(argv[2]);
			case 2: loop1_cnt = (unsigned long long)atoll(argv[1]);
			default:
				break;
		}
	}

	/* Start loop1_func */
	if (loop1_func(loop1_cnt, &sec, &usec) == 0)
		printf("Elapse: %d.%06d sec\n", sec, usec);
	else {
		printf("Encountered error in loop1_func.\n");
		goto err_out;
	}

	/* Start loop2_func */
	if (loop2_func(loop2_cnt, &sec, &usec) == 0)
		printf("Elapse: %d.%06d sec\n", sec, usec);
	else {
		printf("Encountered error in loop2_func.\n");
		goto err_out;
	}

	return 0;

err_out:
	return -1;	

usage:
	printf("usage: %s [loop1_count [loop2_count]]\n", argv[0]);
	return -1;
}

