#############################################################
#
# mtd provides jffs2 utilities
#
#############################################################
#MTD_SOURCE=mtd_20050122.orig.tar.gz
# Use mtd_20070206.tar.gz, it is the same as mtd_20050122.orig.tar but with
# patches from http://uclibc.org/cgi-bin/viewcvs.cgi/trunk/buildroot/package/mtd/mtd.patch 
# applied until to 2007/02/06
#
MTD_VER:=$(strip $(subst ",, $(BR2_MTD_VERSION)))
#"))
MTD_SOURCE=mtd_$(MTD_VER).tar.gz

ifeq ($(strip $(DOWNLOAD_SITE_PUBLIC)),y)
#MTD_SITE=http://ftp.debian.org/debian/pool/main/m/mtd
MTD_SITE := $(DOWNLOAD_SITE)
else
MTD_SITE := $(DOWNLOAD_SITE)
endif
#############################################################
#
# Build mkfs.jffs2 for use on the local host system if
# needed by target/jffs2root.
#
############################################################
MKFS_JFFS2 = $(MTD_HOST_DIR)/util/mkfs.jffs2

MTD_HOST_DIR:=$(TOOL_BUILD_DIR)/mtd-20070206

$(DL_DIR)/$(MTD_SOURCE):
	@[ -f $@ ] || $(WGET) -P $(@D) $(MTD_SITE)/$(@F)

$(MTD_HOST_DIR)/.unpacked: $(DL_DIR)/$(MTD_SOURCE)
	@[ -d $(@D) ] || \
	(zcat $< | tar -C $(TOOL_BUILD_DIR) $(TAR_OPTIONS) -; \
	touch $@; \
	)

$(MTD_HOST_DIR)/util/mkfs.jffs2: $(MTD_HOST_DIR)/.unpacked
	CFLAGS=-I$(LINUX_HEADERS_DIR)/include \
		$(MAKE) LINUXDIR=$(LINUX_DIR) -C $(<D)/util mkfs.jffs2

mtd-host: $(MKFS_JFFS2)

mtd-host-source: $(DL_DIR)/$(MTD_SOURCE)

mtd-host-clean:
	@$(MAKE) -C $(MTD_HOST_DIR) clean

mtd-host-dirclean:
	@[ ! -d $(MTD_HOST_DIR) ] || rm -rf $(MTD_HOST_DIR)

#############################################################
#
# build mtd for use on the target system
#
#############################################################
MTD_DIR:=$(BUILD_DIR)/mtd-20070206

$(MTD_DIR)/.unpacked: $(DL_DIR)/$(MTD_SOURCE)
	@[ -d $(@D) ] || \
	(zcat $< | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -; \
	touch $@; \
	)

MTD_TARGETS_n :=
MTD_TARGETS_y :=

MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASH_ERASE)		+= flash_erase
MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASH_ERASEALL)		+= flash_eraseall
MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASH_INFO)		+= flash_info
MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASH_LOCK)		+= flash_lock
MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASH_UNLOCK)		+= flash_unlock
MTD_TARGETS_$(BR2_PACKAGE_MTD_FLASHCP)			+= flashcp
MTD_TARGETS_$(BR2_PACKAGE_MTD_MKFSJFFS2)		+= mkfs.jffs2
MTD_TARGETS_$(BR2_PACKAGE_MTD_MKFSJFFS)			+= mkfs.jffs
MTD_TARGETS_$(BR2_PACKAGE_MTD_JFFS2DUMP)		+= jffs2dump
MTD_TARGETS_$(BR2_PACKAGE_MTD_SUMTOOL)			+= sumtool
MTD_TARGETS_$(BR2_PACKAGE_MTD_FTL_CHECK)		+= ftl_check
MTD_TARGETS_$(BR2_PACKAGE_MTD_FTL_FORMAT)		+= ftl_format
MTD_TARGETS_$(BR2_PACKAGE_MTD_NFTLDUMP)			+= nftldump
MTD_TARGETS_$(BR2_PACKAGE_MTD_NFTL_FORMAT)		+= nftl_format
MTD_TARGETS_$(BR2_PACKAGE_MTD_NANDDUMPP)		+= nanddump
MTD_TARGETS_$(BR2_PACKAGE_MTD_NANDWRITE)		+= nandwrite
MTD_TARGETS_$(BR2_PACKAGE_MTD_MTD_DEBUG)		+= mtd_debug
MTD_TARGETS_$(BR2_PACKAGE_MTD_DOCFDISK)			+= docfdisk
MTD_TARGETS_$(BR2_PACKAGE_MTD_DOC_LOADBIOS)		+= doc_loadbios

MTD_BUILD_TARGETS := $(addprefix $(MTD_DIR)/util/, $(MTD_TARGETS_y))

$(MTD_BUILD_TARGETS): $(MTD_DIR)/.unpacked
	@CFLAGS="$(TARGET_CFLAGS)" \
	LDFLAGS="$(TARGET_CFLAGS)" \
	$(MAKE)	CC=$(TARGET_CC) LINUXDIR=$(LINUX_DIR) -C $(<D)/util

MTD_TARGETS := $(addprefix $(TARGET_DIR)/usr/sbin/, $(MTD_TARGETS_y))

$(MTD_TARGETS): $(TARGET_DIR)/usr/sbin/% : $(MTD_DIR)/util/%
	@cp -f $< $@
ifeq '$(filter debug,$(COMPILKIND))' ''
	@-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $@
endif

$(TARGET_DIR)/usr/sbin:
	@[ -d $@ ] || mkdir -p $@

ZLIB_DIR:=$(BASE_DIR)/package/share-libs/zlib
$(TARGET_DIR)/lib/libz.so $(ROOTFS_TMP_DIR)/include/zlib.h: Makefile
	@$(MAKE) distclean -C $(ZLIB_DIR) -f $<; \
	$(MAKE) -C $(ZLIB_DIR) -f $<; \
	touch -c $@

mtd: $(TARGET_DIR)/lib/libz.so $(ROOTFS_TMP_DIR)/include/zlib.h $(TARGET_DIR)/usr/sbin $(MTD_TARGETS)

mtd-source: $(DL_DIR)/$(MTD_SOURCE)

.PHONY: mtd-clean mtd-uninstall mtd-install

mtd-install: mtd

mtd-uninstall:
	@$(RM) $(MTD_TARGETS)

mtd-clean: mtd-uninstall
	@[ ! -d $(MTD_DIR) ] || $(MAKE) -C $(MTD_DIR) clean

.PHONY: mtd-dirclean
mtd-dirclean: mtd-uninstall
	@-$(RM) -r $(MTD_DIR)
