#############################################################
#
# bash
#
#############################################################
BASH_VER:=$(strip $(subst ",, $(BR2_BASH_VERSION)))
#"))
BASH_SOURCE:=bash-$(BASH_VER).tar.gz
ifeq ($(strip $(DOWNLOAD_SITE_PUBLIC)),y)
BASH_SITE:=ftp://ftp.gnu.org/gnu/bash
else
BASH_SITE := $(DOWNLOAD_SITE)
endif
BASH_CAT:=zcat
BASH_DIR:=$(BUILD_DIR)/bash-$(BASH_VER)
BASH_BINARY:=bash
BASH_TARGET_BINARY:=bin/bash
FILELIST=$(shell find . -perm 600)

$(DL_DIR)/$(BASH_SOURCE):
	@[ -f $@ ] || $(WGET) -P $(@D) $(BASH_SITE)/$(@F)

bash-source: $(DL_DIR)/$(BASH_SOURCE)

$(BASH_DIR)/.unpacked: $(DL_DIR)/$(BASH_SOURCE)
	@$(BASH_CAT) $< | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -
	@cd $(TOPDIR);  $(TOPDIR)/toolchain/patch-kernel.sh $(@D) $(SUBDIR)/$(BASH_VER) \bash??-???
	touch $@

$(BASH_DIR)/.configured: $(BASH_DIR)/.unpacked
	@(cd $(BASH_DIR); rm -rf config.cache; \
		$(TARGET_CONFIGURE_OPTS) CC_FOR_BUILD=$(HOSTCC) \
		CFLAGS="$(TARGET_CFLAGS)" \
		./configure \
		--target=$(GNU_TARGET_NAME) \
		--host=$(GNU_TARGET_NAME) \
		--build=$(GNU_HOST_NAME) \
		--prefix=/usr \
		--exec-prefix=/usr \
		--bindir=/usr/bin \
		--sbindir=/usr/sbin \
		--libexecdir=/usr/lib \
		--sysconfdir=/etc \
		--datadir=/usr/share \
		--localstatedir=/var \
		--mandir=/usr/man \
		--infodir=/usr/info \
		$(DISABLE_NLS) \
		$(DISABLE_LARGEFILE) \
		--with-curses \
		--enable-alias \
		--without-bash-malloc \
	); \
	touch  $@

$(BASH_DIR)/$(BASH_BINARY): $(BASH_DIR)/.configured
	@$(MAKE) CC=$(TARGET_CC) CC_FOR_BUILD=$(HOSTCC) -C $(BASH_DIR)

$(TARGET_DIR)/$(BASH_TARGET_BINARY): $(BASH_DIR)/$(BASH_BINARY)
	@if [ -f $(@D) ] ; then \
		rm -f $(@D); \
	fi; \
	[ -d $(@D) ] || mkdir -p $(@D)
	@-rm -f $@
	@cp -f $< $@
	@-rm -f $(TARGET_DIR)/bin/sh
	@(cd $(@D); ln -fs $(@F) sh)
ifeq '$(filter debug,$(COMPILKIND))' ''
	@-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $@
endif

bash: $(TARGET_DIR)/$(BASH_TARGET_BINARY)
	@[ -z "$(FILELIST)" ] || chmod 644 $(FILELIST)

.PHONY: bash-clean bash-uninstall bash-install bash-dirclean

bash-install: bash

bash-uninstall:
	@-rm -f $(TARGET_DIR)/$(BASH_TARGET_BINARY)
	@-rm -f $(TARGET_DIR)/bin/sh

bash-clean: bash-uninstall
	@[ ! -d $(BASH_DIR) ] || $(MAKE) -C $(BASH_DIR) clean

bash-dirclean: bash-uninstall
	@-rm -rf $(BASH_DIR)
