#############################################################
#
# busybox
#
#############################################################
BUSYBOX_VER:=$(strip $(subst ",, $(BR2_BUSYBOX_VERSION)))
#"))
BUSYBOX_DIR:=$(BUILD_DIR)/busybox-$(BUSYBOX_VER)
BUSYBOX_SOURCE:=busybox-$(BUSYBOX_VER).tar.bz2
ifeq ($(strip $(DOWNLOAD_SITE_PUBLIC)),y)
BUSYBOX_SITE:=http://www.busybox.net/downloads
else
BUSYBOX_SITE := $(DOWNLOAD_SITE)
endif
BUSYBOX_UNZIP=bzcat
BUSYBOX_CONFIG_FILE=$(TOPDIR)/$(SUBDIR)/busybox.config-$(BUSYBOX_VER)

$(DL_DIR)/$(BUSYBOX_SOURCE):
	@[ -f $@ ] || $(WGET) -P $(@D) $(BUSYBOX_SITE)/$(@F)

busybox-source: $(DL_DIR)/$(BUSYBOX_SOURCE)

$(BUSYBOX_DIR)/.unpacked: $(DL_DIR)/$(BUSYBOX_SOURCE)
	@$(BUSYBOX_UNZIP) $< | tar -C $(BUILD_DIR) $(TAR_OPTIONS) -
	# Allow busybox patches. (only on a certain version.)
	@[ -f $@ ] || \
	(cd $(TOPDIR); $(TOPDIR)/toolchain/patch-kernel.sh $(@D) $(SUBDIR) busybox-$(BUSYBOX_VER)-\*.patch; \
	touch $@; \
	)

$(BUSYBOX_DIR)/.configured: $(BUSYBOX_DIR)/.unpacked
	@cp $(BUSYBOX_CONFIG_FILE) $(@D)/.config; \
	$(SED) s,^CONFIG_PREFIX=.*,CONFIG_PREFIX=\"$(TARGET_DIR)\", $(@D)/.config
ifeq ($(BR2_ENABLE_LOCALE),y)
	@$(SED) "s/^.*CONFIG_LOCALE_SUPPORT.*/CONFIG_LOCALE_SUPPORT=y/;" $(@D)/.config
else
	@$(SED) "s/^.*CONFIG_LOCALE_SUPPORT.*/CONFIG_LOCALE_SUPPORT=n/;" $(@D)/.config
endif
ifeq ($(BR2_BUSYBOX_VERSION_1_0_0),y)
	@$(SED) "s,^CROSS.*,CROSS=$(TARGET_CROSS)\n\
		PREFIX=$(TARGET_DIR),;" $(@D)/Rules.mak
endif
ifeq ($(BR2_BUSYBOX_VERSION_1_2_2_1),y)
	@$(SED) "s,^CROSS.*,CROSS=$(TARGET_CROSS)\n\
		PREFIX=$(TARGET_DIR),;" $(@D)/Rules.mak
endif
ifeq ($(BR2_LARGEFILE),y)
	@$(SED) "s/^.*CONFIG_LFS.*/CONFIG_LFS=y/;" $(@D)/.config
else
	@$(SED) "s/^.*CONFIG_LFS.*/CONFIG_LFS=n/;" $(@D)/.config
	@$(SED) "s/^.*FDISK_SUPPORT_LARGE_DISKS.*/FDISK_SUPPORT_LARGE_DISKS=n/;" $(@D)/.config
endif
ifeq ($(BR2_BUSYBOX_CUSTOMIZATION),y)
	@[ -f $@ ] || \
	($(MAKE) CC=$(TARGET_CC) CROSS_COMPILE="$(TARGET_CROSS)" CROSS="$(TARGET_CROSS)" -C $(@D) menuconfig; \
	touch $@; \
	)
else
	@[ -f $@ ] || \
	($(MAKE) CC=$(TARGET_CC) CROSS_COMPILE="$(TARGET_CROSS)" CROSS="$(TARGET_CROSS)" -C $(@D) oldconfig; \
	touch $@; \
	)
endif

.PHONY: busybox-configured
busybox-configured: $(BUSYBOX_DIR)/.configured

$(BUSYBOX_DIR)/busybox: $(BUSYBOX_DIR)/.configured
	@$(MAKE) CC=$(TARGET_CC) CROSS_COMPILE="$(TARGET_CROSS)" CROSS="$(TARGET_CROSS)" CONFIG_PREFIX="$(TARGET_DIR)" \
		EXTRA_CFLAGS="$(TARGET_CFLAGS)" -C $(<D)

$(TARGET_DIR)/bin/busybox: $(BUSYBOX_DIR)/busybox
	@[ ! -f $(@D) ] || rm -f $(@D)
	@[ -d $(@D) ] || mkdir -p $(@D)
	@[ -d $(@D)/../usr/local/bin ] || mkdir -p $(@D)/../usr/local/bin
	@-rm -f $(@D)/awk
	@$(MAKE) CC=$(TARGET_CC) CROSS_COMPILE="$(TARGET_CROSS)" CROSS="$(TARGET_CROSS)" CONFIG_PREFIX="$(TARGET_DIR)" \
		EXTRA_CFLAGS="$(TARGET_CFLAGS)" -C $(<D) install; \
	[ -d $(TARGET_DIR)/usr/share/udhcpc ] || mkdir -p $(TARGET_DIR)/usr/share/udhcpc; \
	cp -f $(TOPDIR)/target/generic/target_skeleton/usr/share/udhcpc/default.script $(TARGET_DIR)/usr/share/udhcpc/.; \
	chmod a+x $(TARGET_DIR)/usr/share/udhcpc/default.script
ifeq '$(filter debug,$(COMPILKIND))' ''
	@-$(STRIPCMD) $(STRIP_STRIP_UNNEEDED) $@
endif

busybox:  $(TARGET_DIR)/bin/busybox

.PHONY: busybox-clean busybox-uninstall busybox-install busybox-dirclean

busybox-install: busybox

busybox-uninstall:
	@[ ! -d $(BUSYBOX_DIR) ] || $(MAKE) -C $(BUSYBOX_DIR) uninstall
	@-rm -f $(TARGET_DIR)/usr/share/udhcpc/default.script

busybox-clean: busybox-uninstall
	@[ ! -d $(BUSYBOX_DIR) ] || $(MAKE) -C $(BUSYBOX_DIR) clean

busybox-dirclean: busybox-uninstall
	@-rm -rf $(BUSYBOX_DIR)
