#!/bin/bash

#  
#  Copyright (c) 2010-2011, Intel Corporation
#  All rights reserved.
#  
#  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
#  
#      * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#      * Neither the name of Intel Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  

export ROOTFS_DIR=cs_rootfs_1.2.12
export KERNEL_DIR=smp86xx_kernel_source_R2.6.22-36
export UCLINUX_KERNEL=`pwd`/$KERNEL_DIR/linux-2.6.22.19

# Set Toolchain dir
if [ -f mips-4.3/bin/mips-linux-gnu-gcc ]; then
  export SMP86XX_TOOLCHAIN_PATH=`pwd`/mips-4.3
else
  if [ -f mips-4.3-51-mips-linux-gnu-i686-pc-linux-gnu.tar.bz2 ]; then
    tar xjvf mips-4.3-51-mips-linux-gnu-i686-pc-linux-gnu.tar.bz2
    export SMP86XX_TOOLCHAIN_PATH=`pwd`/mips-4.3
  fi
fi

# Make sure SMP86XX_TOOLCHAIN_PATH is set
if [ "$SMP86XX_TOOLCHAIN_PATH" == "" ]; then
  echo ERROR: SMP86XX_TOOLCHAIN_PATH is not set
  return 1
fi

# Source toolchain 
export PATH=$SMP86XX_TOOLCHAIN_PATH/bin:$PATH

# Set build flags
export COMPILKIND="glibc hardfloat codesourcery release"

# Create rootfs-path.env
echo TOPDIR=\"`pwd`/${ROOTFS_DIR}\">${ROOTFS_DIR}/rootfs-path.env
echo export TOPDIR>>${ROOTFS_DIR}/rootfs-path.env
echo SMP86XX_ROOTFS_PATH=\"`pwd`/${ROOTFS_DIR}\">>${ROOTFS_DIR}/rootfs-path.env
echo export SMP86XX_ROOTFS_PATH>>${ROOTFS_DIR}/rootfs-path.env
echo SMP86XX_TOOLCHAIN_PATH=\"$SMP86XX_TOOLCHAIN_PATH\">>${ROOTFS_DIR}/rootfs-path.env
echo export SMP86XX_TOOLCHAIN_PATH>>${ROOTFS_DIR}/rootfs-path.env
echo COMPILKIND=\"$COMPILKIND\">>${ROOTFS_DIR}/rootfs-path.env
echo export COMPILKIND>>${ROOTFS_DIR}/rootfs-path.env
echo INSTALL_MOD_PATH=\"`pwd`/${ROOTFS_DIR}/build_mipsel/root\">>${ROOTFS_DIR}/rootfs-path.env
echo export INSTALL_MOD_PATH>>${ROOTFS_DIR}/rootfs-path.env
echo PATH=`pwd`/${ROOTFS_DIR}/host/bin:`pwd`/${ROOTFS_DIR}/cross_rootfs/bin:$PATH>>${ROOTFS_DIR}/rootfs-path.env
echo export PATH>>${ROOTFS_DIR}/rootfs-path.env

# Create .makefile.in
echo TOPDIR=`pwd`/${ROOTFS_DIR}>${ROOTFS_DIR}/.makefile.in
echo export TOPDIR>>${ROOTFS_DIR}/.makefile.in
echo SMP86XX_ROOTFS_PATH=`pwd`/${ROOTFS_DIR}>>${ROOTFS_DIR}/.makefile.in
echo export SMP86XX_ROOTFS_PATH>>${ROOTFS_DIR}/.makefile.in
echo SMP86XX_TOOLCHAIN_PATH=$SMP86XX_TOOLCHAIN_PATH>>${ROOTFS_DIR}/.makefile.in
echo export SMP86XX_TOOLCHAIN_PATH>>${ROOTFS_DIR}/.makefile.in
echo COMPILKIND=$COMPILKIND>>${ROOTFS_DIR}/.makefile.in
echo export COMPILKIND>>${ROOTFS_DIR}/.makefile.in
echo INSTALL_MOD_PATH=`pwd`/${ROOTFS_DIR}/build_mipsel/root>>${ROOTFS_DIR}/.makefile.in
echo export INSTALL_MOD_PATH>>${ROOTFS_DIR}/.makefile.in
echo PATH=`pwd`/${ROOTFS_DIR}/host/bin:`pwd`/${ROOTFS_DIR}/cross_rootfs/bin:$PATH>>${ROOTFS_DIR}/.makefile.in
echo export PATH>>${ROOTFS_DIR}/.makefile.in

# Source rootfs
pushd ${ROOTFS_DIR} &>/dev/null
source rootfs-path.env 
popd &>/dev/null

